@@ -948,6 +948,21 @@ def test_reduce_agg(v):
     assert s.is_empty
 
 
+def test_reduce_agg_count_is_int64(v):
+    """Aggregators that count should default to INT64 return dtype."""
+    assert v.dtype == dtypes.INT64
+    res = v.reduce(agg.count).new()
+    assert res.dtype == dtypes.INT64
+    assert res == 4
+    res = v.dup(dtypes.INT8).reduce(agg.count).new()
+    assert res.dtype == dtypes.INT64
+    assert res == 4
+    # Allow return dtype to be specified
+    res = v.dup(dtypes.INT8).reduce(agg.count[dtypes.INT16]).new()
+    assert res.dtype == dtypes.INT16
+    assert res == 4
+
+
 @pytest.mark.skipif("not suitesparse")
 def test_reduce_agg_argminmax(v):
     assert v.reduce(agg.ss.argmin).new() == 6
@@ -1532,6 +1547,8 @@ def test_outer(v):
 
 @autocompute
 def test_auto(v):
+    from graphblas.core.infix import VectorEwiseMultExpr
+
     v = v.dup(dtype=bool)
     expected = binary.land(v & v).new()
     assert 0 not in expected
@@ -1579,16 +1596,26 @@ def test_auto(v):
             "__rand__",
             "__ror__",
         ]:
+            # print(type(expr).__name__, method)
             val1 = getattr(expected, method)(expected).new()
-            val2 = getattr(expected, method)(expr)
-            val3 = getattr(expr, method)(expected)
-            val4 = getattr(expr, method)(expr)
-            assert val1.isequal(val2)
-            assert val1.isequal(val3)
-            assert val1.isequal(val4)
-            assert val1.isequal(val2.new())
-            assert val1.isequal(val3.new())
-            assert val1.isequal(val4.new())
+            if method in {"__or__", "__ror__"} and type(expr) is VectorEwiseMultExpr:
+                # Doing e.g. `plus(x & y | z)` isn't allowed--make user be explicit
+                with pytest.raises(TypeError):
+                    val2 = getattr(expected, method)(expr)
+                with pytest.raises(TypeError):
+                    val3 = getattr(expr, method)(expected)
+                with pytest.raises(TypeError):
+                    val4 = getattr(expr, method)(expr)
+            else:
+                val2 = getattr(expected, method)(expr)
+                assert val1.isequal(val2)
+                assert val1.isequal(val2.new())
+                val3 = getattr(expr, method)(expected)
+                assert val1.isequal(val3)
+                assert val1.isequal(val3.new())
+                val4 = getattr(expr, method)(expr)
+                assert val1.isequal(val4)
+                assert val1.isequal(val4.new())
         s1 = expected.reduce(monoid.lor).new()
         s2 = expr.reduce(monoid.lor)
         assert s1.isequal(s2.new())
@@ -1652,7 +1679,7 @@ def test_expr_is_like_vector(v):
         "resize",
         "update",
     }
-    ignore = {"__sizeof__"}
+    ignore = {"__sizeof__", "_ewise_add", "_ewise_mult", "_ewise_union", "_inner", "_vxm"}
     assert attrs - expr_attrs - ignore == expected, (
         "If you see this message, you probably added a method to Vector.  You may need to "
         "add an entry to `vector` or `matrix_vector` set in `graphblas.core.automethods` "
@@ -1701,7 +1728,7 @@ def test_index_expr_is_like_vector(v):
         "from_values",
         "resize",
     }
-    ignore = {"__sizeof__"}
+    ignore = {"__sizeof__", "_ewise_add", "_ewise_mult", "_ewise_union", "_inner", "_vxm"}
     assert attrs - expr_attrs - ignore == expected, (
         "If you see this message, you probably added a method to Vector.  You may need to "
         "add an entry to `vector` or `matrix_vector` set in `graphblas.core.automethods` "
